package com.github.jfcloud.gene.sample.service.biz;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.Assert;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.jfcloud.gene.common.constant.WhetherEnum;
import com.github.jfcloud.gene.common.util.CustomIdGenerator;
import com.github.jfcloud.gene.file.entity.FileInfo;
import com.github.jfcloud.gene.file.service.FileInfoService;
import com.github.jfcloud.gene.file.vo.FileVo;
import com.github.jfcloud.gene.sample.entity.PathologicalPreExperiment;
import com.github.jfcloud.gene.sample.entity.SamplePathological;
import com.github.jfcloud.gene.sample.mapper.SamplePathologicalMapper;
import com.github.jfcloud.gene.sample.service.PathologicalPreExperimentService;
import com.github.jfcloud.gene.sample.service.SampleEditService;
import com.github.jfcloud.gene.sample.service.SamplePathologicalService;
import com.github.jfcloud.gene.sample.vo.PreExperimentVo;
import com.github.jfcloud.gene.sample.vo.SamplePathologicalVo;
import com.github.jfcloud.gene.sample.vo.SampleSubmitVo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class SamplePathologicalServiceImpl extends ServiceImpl<SamplePathologicalMapper, SamplePathological> implements SamplePathologicalService, SampleEditService {

    /**
     * 病理样本-样本信息附件类型
     */
    public static final String PATHOLOGICAL_SAMPLE_INFO = "pathological.sampleInfo";
    /**
     * 病理样本-抗体信息附件类型
     */
    public static final String PATHOLOGICAL_ANTIBODY_INFO = "pathological.antibodyInfo";
    private final PathologicalPreExperimentService preExperimentService;
    private final FileInfoService fileInfoService;

    @Override
    public void removeBySampleId(Long sampleId) {
        List<SamplePathological> pathologicalList = list(new LambdaQueryWrapper<>(SamplePathological.class)
                .select(SamplePathological::getId)
                .eq(SamplePathological::getSampleId, sampleId));
        if (pathologicalList.isEmpty()) {
            return;
        }

        Set<Long> idSet = pathologicalList.stream().map(SamplePathological::getId).collect(Collectors.toSet());
        removeBatchByIds(idSet);

        preExperimentService.remove(new LambdaQueryWrapper<>(PathologicalPreExperiment.class).in(PathologicalPreExperiment::getPathologicalId, idSet));
        fileInfoService.remove(new LambdaQueryWrapper<>(FileInfo.class).in(FileInfo::getRelateId, idSet));
    }

    @Override
    public void save(Long sampleId, SampleSubmitVo vo) {
        SamplePathologicalVo pathological = vo.getPathological();
        Assert.notNull(pathological, "病理样本送检不能为空");
        removeBySampleId(sampleId);

        //保存病理样本送检信息
        SamplePathological samplePathological = BeanUtil.copyProperties(pathological, SamplePathological.class);
        samplePathological.setId(CustomIdGenerator.nextId());
        samplePathological.setSampleId(sampleId);
        //清空预实验相关的参数
        if (!pathological.getPreExperiment()) {
            samplePathological.setPreDate("")
                    .setPrePeople("")
                    .setPreSubjectName("")
                    .setPreApplicantId(null)
                    .setPreApplicantName("")
                    .setPreApplicantLeaderId(null)
                    .setPreApplicantLeaderName("")
                    .setPreContent("")
                    .setPreConclusion("");
        }
        samplePathological.insert();

        //保存预实验列表
        if (pathological.getPreExperiment()) {
            List<PathologicalPreExperiment> preExperiments = BeanUtil.copyToList(pathological.getExperiments(), PathologicalPreExperiment.class);
            preExperiments.forEach(item -> item.setPathologicalId(samplePathological.getId()));
            preExperimentService.saveBatch(preExperiments);
        }

        //保存附件列表
        List<FileInfo> fileInfos = new ArrayList<>();
        pathological.getSampleInfoFiles().forEach(f -> {
                    FileInfo fileInfo = new FileInfo()
                            .setRelateId(samplePathological.getId())
                            .setType(PATHOLOGICAL_SAMPLE_INFO)
                            .setName(f.getName())
                            .setUrl(f.getUrl());
                    fileInfos.add(fileInfo);
                });
        pathological.getAntibodyInfoFiles().forEach(f -> {
                    FileInfo fileInfo = new FileInfo()
                            .setRelateId(samplePathological.getId())
                            .setType(PATHOLOGICAL_ANTIBODY_INFO)
                            .setName(f.getName())
                            .setUrl(f.getUrl());
                    fileInfos.add(fileInfo);
                });
        if (CollUtil.isNotEmpty(fileInfos)) {
            fileInfoService.saveBatch(fileInfos);
        }

        log.info("保存病理样本送检信息成功");
        generate(sampleId);
    }

    @Override
    public void generate(Long sampleId) {

    }

    @Override
    public SamplePathologicalVo getDetail(Long sampleId) {
        SamplePathological samplePathological = getOne(new LambdaQueryWrapper<>(SamplePathological.class)
                .eq(SamplePathological::getSampleId, sampleId)
                .eq(SamplePathological::getDeleted, WhetherEnum.NO.getCode()));
        if (samplePathological == null) {
            return null;
        }

        SamplePathologicalVo pathologicalVo = BeanUtil.copyProperties(samplePathological, SamplePathologicalVo.class);

        //预实验列表
        List<PathologicalPreExperiment> preExperiments = preExperimentService.list(new LambdaQueryWrapper<>(PathologicalPreExperiment.class)
                .eq(PathologicalPreExperiment::getPathologicalId, samplePathological.getId()));
        pathologicalVo.getExperiments().addAll(BeanUtil.copyToList(preExperiments, PreExperimentVo.class));

        //附件列表
        List<FileInfo> fileInfos = fileInfoService.list(new LambdaQueryWrapper<>(FileInfo.class)
                .eq(FileInfo::getRelateId, samplePathological.getId())
                .eq(FileInfo::getDeleted, WhetherEnum.NO.getCode())
                .orderByAsc(FileInfo::getId));
        if (CollUtil.isNotEmpty(fileInfos)) {
            pathologicalVo.getSampleInfoFiles().addAll(fileInfos.stream()
                    .filter(item -> PATHOLOGICAL_SAMPLE_INFO.equals(item.getType()))
                    .map(item -> BeanUtil.copyProperties(item, FileVo.class))
                    .collect(Collectors.toList()));

            pathologicalVo.getAntibodyInfoFiles().addAll(fileInfos.stream()
                    .filter(item -> PATHOLOGICAL_ANTIBODY_INFO.equals(item.getType()))
                    .map(item -> BeanUtil.copyProperties(item, FileVo.class))
                    .collect(Collectors.toList()));
        }

        return pathologicalVo;
    }
}